#include "stdafx.h"

#pragma comment(lib,"pdcurses.lib")

#include <algorithm>
#include "curses.h"
#include <granite.h>
#include "letters.h"
using namespace Granite::System;

int W, H; // rozmiary konsoli
int ofx=0,ofy=0;
int lockx=0,locky=0;
bool lockSteering=false;
RandomGen rg;

char current_char='a';
char current_charL='A';

bool cput(int x,int y){
	int ox=ofx+x;
	int oy=ofy+y;
	if(ox>=0&&oy>=0&&ox<H&&oy<W)
		return true;
	return false;
}
void put(int x,int y,char c){
	int ox=ofx+x;
	int oy=ofy+y;
	if(ox>=0&&oy>=0&&ox<H&&oy<W)
		mvaddch(ox,oy,c);
}
void putnooff(int x,int y,char c){
	int ox=x;
	int oy=y;
	if(ox>=0&&oy>=0&&ox<H&&oy<W)
		mvaddch(ox,oy,c);
}
void putsnooff(int x,int y,const string &c){
	int ox=x;
	int oy=y;
	for(int i=0;i<c.size();++i){
		if(ox>=0&&oy>=0&&ox<H&&oy<W)
			mvaddch(ox,oy+i,c[i]);
	}
}

class gfxTaskCmp;
class gfxTask{
	protected:
		void updateTimers(float dt){
			t+=dt;
			tend-=dt;
			if(tend<0.f)
				del=true;
		}
		int sort;
	public:
		float t; // licznik (aktualny czas zadania)
		float tend; // limit czasowy na wykonanie
		bool del; // znacznik usuwania
		gfxTask(int s):t(0.f),tend(6.f),del(false),sort(s){}
		virtual ~gfxTask(){}
		virtual void update(float dt){}
		void setTime(float et){tend=et;}
	friend class gfxTaskCmp;
};
class gfxTaskCmp{
	public:
		bool operator()(const gfxTask *a,const gfxTask *b){
			return a->sort<b->sort;
		}
};

std::list<gfxTask*> tasks;
void clearTasks(){
	std::vector<std::list<gfxTask*>::iterator> todel;
	for(std::list<gfxTask*>::iterator it=tasks.begin();it!=tasks.end();++it)
		if((*it)->del)
			todel.push_back(it);
	for(int i=0;i<todel.size();++i){
		delete *todel[i];
		tasks.erase(todel[i]);
	}
}

class gfxTaskExplosion:public gfxTask{
	public:
		Vec2f center;
		std::vector<Vec2f> p; // czasteczki
		std::vector<float> pa; // katy
		char character;
		int color;
		float speed;
		gfxTaskExplosion(const Vec2f &c,char ch,int count,int col,float s):center(c),character(ch),color(col),speed(s),gfxTask(7){
			for(int i=0;i<count;++i){
				p.push_back(Vec2f(0.f,0.f));
				pa.push_back(rg.Float(GE_PI_D2,GE_2PI+GE_PI_D2));
			}
		}
		virtual void update(float dt){
			attron(COLOR_PAIR(color+5)| A_NORMAL);
			dt*=speed;
			updateTimers(dt);
			for(int i=0;i<p.size();++i){
				p[i].x=Cosf(pa[i])*t;
				p[i].y=-Sinf(pa[i])*t;
				if(pa[i]<GE_PI+GE_PI_D2)
					pa[i]+=dt*0.05f;
				if(pa[i]>GE_PI+GE_PI_D2)
					pa[i]-=dt*0.05f;
				
				int x=int(center.x)+int(p[i].x);
				int y=int(center.y)+int(p[i].y);
				put(y,x,character);
			}
		}
};
class gfxTaskFireworks:public gfxTask{
	public:
		Vec2f center;
		std::vector<Vec2f> p; // czasteczki
		std::vector<float> pa; // katy
		char character;
		int color;
		float speed;
		gfxTaskFireworks(const Vec2f &c,char ch,int count,int col,float s):center(c),character(ch),color(col),speed(s),gfxTask(7){
			for(int i=0;i<count;++i){
				p.push_back(Vec2f(0.f,0.f));
				pa.push_back(rg.Float(GE_PI_D2,GE_2PI+GE_PI_D2));
			}
		}
		virtual void update(float dt){
			attron(COLOR_PAIR(color+8)| A_NORMAL);
			dt*=speed;
			updateTimers(dt);
			for(int i=0;i<p.size();++i){
				p[i].x=Cosf(pa[i])*t;
				p[i].y=-Sinf(pa[i])*t;
				if(pa[i]<GE_PI+GE_PI_D2)
					pa[i]+=dt*0.05f;
				if(pa[i]>GE_PI+GE_PI_D2)
					pa[i]-=dt*0.05f;
				
				int x=int(center.x)+int(p[i].x);
				int y=int(center.y)+int(p[i].y);
				put(y,x,character);
			}
		}
};

class gfxStars:public gfxTask{
	public:
		Vec2f center;
		std::vector<Vec2f> p; // czasteczki
		std::vector<float> pa; // katy
		std::vector<float> pt; // czasy
		char character;
		int color;
		float speed;
		gfxStars(const Vec2f &c,char ch,int count,int col,float s):center(c),character(ch),color(col),speed(s),gfxTask(1){
			for(int i=0;i<count;++i){
				p.push_back(Vec2f(0.f,0.f));
				pa.push_back(rg.Float(GE_PI_D2,GE_2PI+GE_PI_D2));
				pt.push_back(rg.Float(0.f,speed*2.f));
			}
		}
		virtual void update(float dt){
			attron(COLOR_PAIR(color)| A_NORMAL);
			dt*=speed;
			//updateTimers(dt); inf
			for(int i=0;i<p.size();++i){
				pt[i]+=dt;
				if(pt[i]>2.f*speed)
					pt[i]=0.f;
				p[i].x=Cosf(pa[i])*pt[i];
				p[i].y=-Sinf(pa[i])*pt[i];
				
				int x=int(center.x)+int(p[i].x);
				int y=int(center.y)+int(p[i].y);
				putnooff(y,x,character);
			}
		}
};
class gfxRandCloud:public gfxTask{
	public:
		Vec2f center;
		float r;
		char character;
		int color;
		int count;
		gfxRandCloud(const Vec2f &c,char ch,int ct,int col,int radial):center(c),character(ch),color(col),count(ct),r(radial),gfxTask(6){}
		virtual void update(float dt){
			attron(COLOR_PAIR(color)| A_NORMAL);
			updateTimers(dt);
			for(int i=0;i<count;++i){
				float l=rg.Float(0.f,r);
				float a=rg.Float(-GE_PI,GE_PI);
				int x=int(center.x)+int(Cosf(a)*l);
				int y=int(center.y)+int(-Sinf(a)*l);
				put(y,x,character);
			}
		}
};
class gfxMissle:public gfxTask{
	public:
		Vec2f pos;
		gfxMissle(const Vec2f &p):pos(p),gfxTask(33){}
		virtual void update(float dt){
			attron(COLOR_PAIR(10)| A_NORMAL);/*
			updateTimers(dt);
			for(int i=0;i<count;++i){
				float l=rg.Float(0.f,r);
				float a=rg.Float(-GE_PI,GE_PI);
				int x=int(center.x)+int(Cosf(a)*l);
				int y=int(center.y)+int(-Sinf(a)*l);
				put(y,x,character);
			}*/
		}
};
class gfxShake:public gfxTask{
	public:
		gfxShake():gfxTask(0){
			lockSteering=true;
			lockx=ofx;
			locky=ofy;
		}
		virtual void update(float dt){
			this->updateTimers(dt);
			if(this->del){
				lockSteering=false;
				ofx=lockx;
				ofy=locky;
			}
			else{
				int off=Min(3,int(tend*2.f));
				ofx=lockx+rg.Int(-off,off);
				ofy=locky+rg.Int(-off,off);
			}
		}
};
class gfxText:public gfxTask{
		string text;
		Vec2i pos;
		int color;
	public:
		gfxText(const string &txt,const Vec2i &position,int cl):pos(position),text(txt),color(cl),gfxTask(666){}
		virtual void update(float dt){
			this->updateTimers(dt);
			attron(COLOR_PAIR(color)| A_NORMAL);
			for(int i=0;i<text.size();++i)
				putnooff(pos.y,pos.x+i,text[i]);
		}
};

void pushExplosion(Vec2i pos,int layers,char ch,bool particles=true,bool colors=true){ // layers [1-3]
	const float time=1.2f; //2
	int count=30;
	const float sm=40.f; // speed modifier
	float speeds[]={1.f,0.85f,0.7f};
	for(int i=0;i<layers;++i){
		gfxTaskExplosion *e=new gfxTaskExplosion(Vec2f(pos.x,pos.y),ch,count,1+(colors?i:0),speeds[i]*sm);
		e->setTime(time*sm);
		tasks.push_back(e);
	}
	if(particles){
		gfxRandCloud *e=new gfxRandCloud(Vec2f(pos.x,pos.y),ch,5,colors?3:2,8);
		e->setTime(time/4.f);
		tasks.push_back(e);
	}
}
void pushFireworks(Vec2i pos,int layers,char ch,bool particles=true,bool colors=true){ // layers [1-3]
	const float time=1.2f; //2
	int count=30;
	const float sm=40.f; // speed modifier
	float speeds[]={1.f,0.85f,0.7f};
	for(int i=0;i<layers;++i){
		gfxTaskFireworks *e=new gfxTaskFireworks(Vec2f(pos.x,pos.y),ch,count,1+(colors?i:0),speeds[i]*sm);
		e->setTime(time*sm);
		tasks.push_back(e);
	}
	if(particles){
		gfxRandCloud *e=new gfxRandCloud(Vec2f(pos.x,pos.y),ch,5,colors?3:2,8);
		e->setTime(time/4.f);
		tasks.push_back(e);
	}
}
void pushStars(Vec2i pos,char ch,int color){
	const float time=2.f;
	int count=60;
	const float sm=40.f;
	gfxStars *e=new gfxStars(Vec2f(pos.x,pos.y),ch,count,5,sm);
	e->setTime(time*sm);
	tasks.push_back(e);
}
void pushShake(float t){
	gfxShake *e=new gfxShake();
	e->setTime(t);
	tasks.push_back(e);
}
void pushText(Vec2i pos,string s,int color,float time){
	gfxText *e=new gfxText(s,pos,color);
	e->setTime(time);
	tasks.push_back(e);
}
/*
void jLine(int x1,int x2,int y1,int y2){
	for(float a=0.f;a<=1.f;a+=0.01f){
		float x=float(x1)+float(x2-x1)*a;
		float y=float(y1)+float(y2-y1)*a;
		put(int(x),int(y),'Z');
	}
}*/

float playerLife=1.f;
char *letter=letters_a;
char current_letter[32*32];
int letter_x,letter_y;
void transformL(float a,float s){
	Mat44f m;
	m.SetIdentity();
	m.SetRotation(a,Vec3f(1.f,0.f,0.f));
	m.Scale(Vec3f(s,s,s));
	memset(current_letter,0,sizeof(current_letter));
	int i=0;
	for(int x=0;x<32;++x)
		for(int y=0;y<32;++y){
			if(letter[i++]==1){
				Vec3f in(x-16,y-16,0.f),out;
				m.Transform(in,out);
				int ox=int(out.x)+16;
				int oy=int(out.y)+16;
				if(ox>=32||ox<0||oy>=32||oy<0)
					continue;
				current_letter[ox*32+oy]=1;
			}
		}
}
void jDrawLetter(){
	int xo=letter_x;
	int yo=letter_y;
	int i=0;
	for(int x=xo;x<xo+32;++x)
		for(int y=yo;y<yo+32;++y)
			put(x,y,current_letter[i++]==1?current_charL:' ');
}
int startLife=0;
int getOnes(){
	int p=0;
	for(int i=0;i<32*32;++i)
		if(letter[i]==1)
			++p;
	return p;
}
void setLetterStartLife(){
	startLife=getOnes();
}
float getLetterLife(){
	int p=getOnes();
	if(p==0)
		return -1.f;
	return float(p)/float(startLife);
}
bool checkCollision(int x,int y,float gunPower=5.f){ // gunPower 2-5
	int xo=letter_x;
	int yo=letter_y;
	if(x<xo||x>xo+32||y<yo||y>yo+32)
		return false;
	bool hit=current_letter[32*(x-xo)+(y-yo)]==1;
	if(hit){
		int x1=x-xo;
		int y1=y-yo;
		for(int cx=x1-6;cx<x1+6;++cx)
			for(int cy=y1-6;cy<y1+6;++cy){
				if(Vec2i(cx-x1,cy-y1).Length()<gunPower)
					if(cx*32+cy>0 && cx*32+cy<32*32)
						letter[cx*32+cy]=0;
			}
	}
	return hit;
}
bool checkCollisionBlast(int x,int y,float gunPower=6.f,float randFact=-0.2f){ // gunPower 2-5, randFact - || 0-1
	int xo=letter_x;
	int yo=letter_y;
	if(x<xo||x>xo+32||y<yo||y>yo+32)
		return false;
	bool hit=false;
	int x1=x-xo;
	int y1=y-yo;
	for(int cx=x1-9;cx<x1+9;++cx)
		for(int cy=y1-9;cy<y1+9;++cy){
			if(Vec2i(cx-x1,cy-y1).Length()<gunPower)
				if(cx*32+cy>0 && cx*32+cy<32*32){
					if(letter[cx*32+cy]==1 && (randFact<0.f || rg.FloatClamp()<randFact)){
						hit=true;
						letter[cx*32+cy]=0;
					}
				}
		}
	return hit;
}

float launchMissleTimeA=5.f;
float launchMissleTimeB=10.f;

Vec2i gunOfDeathPos;
bool gunOfDeath=false;
float gunOfDeathTime=30.f; // 30-2*30 sekund
float godActive=10.f;

void drawGunOfDeath(){
	attron(COLOR_PAIR(9)| A_NORMAL);
	for(int x=gunOfDeathPos.x-2;x<gunOfDeathPos.x+2;++x)
		for(int y=gunOfDeathPos.y-2;y<gunOfDeathPos.y+2;++y)
			put(x,y,'G');
}

bool gunPrecise=true;
float gunPower=2.f;
float gunRandFactor=0.2f;
float gunTime=3.f;
float gunFireRate=1.f; // co ile sekund strzal
std::list<std::pair<Vec2i,float>> pMissles; //pociski wystrzelone przez nas
void pushPMissle(Vec2i pos){
	pMissles.push_back(std::pair<Vec2i,float>(pos,gunTime));
}
void nextLetter(bool f=false);
void updatePMissles(float dt){
	attron(COLOR_PAIR(11)| A_NORMAL);
	std::vector<std::list<std::pair<Vec2i,float>>::iterator> todel;
	for(std::list<std::pair<Vec2i,float>>::iterator it=pMissles.begin();it!=pMissles.end();++it){
		if(it->second<0.f){
			todel.push_back(it);
			if(gunOfDeath){
				if(checkCollisionBlast(it->first.x,it->first.y,8.f,-1.f))
					pushExplosion(Vec2i(it->first.y,it->first.x),rg.Int(1,3),current_char);
			}
			else{
				if(gunPrecise){
					if(checkCollision(it->first.x,it->first.y,gunPower))
						pushExplosion(Vec2i(it->first.y,it->first.x),rg.Int(1,3),current_char);
				}
				else{
					if(checkCollisionBlast(it->first.x,it->first.y,gunPower,gunRandFactor))
						pushExplosion(Vec2i(it->first.y,it->first.x),rg.Int(1,3),current_char);
				}
			}
			if(getLetterLife()<0.f){
				nextLetter();
				transformL(0.f,1.0f);
			}
		}
		else{
			put(it->first.x,it->first.y,current_charL);
			if(it->second>(2.f/3.f)*gunTime){
				put(it->first.x+1,it->first.y-1,current_charL);
				put(it->first.x+1,it->first.y+1,current_charL);
				put(it->first.x-1,it->first.y-1,current_charL);
				put(it->first.x-1,it->first.y+1,current_charL);
				
				put(it->first.x,it->first.y-2,current_charL);
				put(it->first.x+2,it->first.y,current_charL);
				put(it->first.x,it->first.y+2,current_charL);
				put(it->first.x-2,it->first.y,current_charL);
			}
			if(it->second>(1.f/3.f)*gunTime){
				put(it->first.x+1,it->first.y,current_charL);
				put(it->first.x,it->first.y+1,current_charL);
				put(it->first.x,it->first.y-1,current_charL);
				put(it->first.x-1,it->first.y,current_charL);
			}
			it->second-=dt;
		}
	}
	for(int i=0;i<todel.size();++i)
		pMissles.erase(todel[i]);
}

void nextLetter(bool first){
	if(letter==letters_a && !first){
		letter=letters_b;
		current_char='b';
		current_charL='B';
		
		gunPower=2.f;
		gunRandFactor=0.2f;
		gunTime=3.f;
		gunFireRate=0.8f;
		pushText(Vec2i(4,1),"Ulepszono fire rate broni (0.8s)",1,2.f);
	}
	else if(letter==letters_b){
		letter=letters_c;
		current_char='c';
		current_charL='C';
		
		gunPower=3.f;
		pushText(Vec2i(4,1),"Zwiekszono moc broni",1,2.f);
	}
	else if(letter==letters_c){
		letter=letters_d;
		current_char='d';
		current_charL='C';
	}
	else if(letter==letters_d){
		letter=letters_e;
		current_char='e';
		current_charL='E';
		launchMissleTimeA=4.f;
		pushText(Vec2i(4,1),"Comicsy zwiekszyly szybkosc ladowania swoich pociskow! (4-10 sekund)",1,2.f);
		
		gunPower=3.f;
		gunFireRate=0.7f;
		pushText(Vec2i(4,2),"Ulepszono fire rate broni (0.7s) i moc broni",1,2.f);
	}
	else if(letter==letters_e){
		letter=letters_f;
		current_char='f';
		current_charL='F';
		
		gunTime=2.f;
		pushText(Vec2i(4,1),"Ulepszono predkosc pocisku (2s)",1,2.f);
	}
	else if(letter==letters_f){
		letter=letters_g;
		current_char='g';
		current_charL='G';
		
		gunPrecise=false;
		pushText(Vec2i(4,1),"Przelaczono bron w tryb shotguna",1,2.f);
	}
	else if(letter==letters_g){
		letter=letters_h;
		current_char='h';
		current_charL='H';
	}
	else if(letter==letters_h){
		letter=letters_i;
		current_char='i';
		current_charL='I';
		
		gunRandFactor=0.3f;
		pushText(Vec2i(4,1),"Zwiekszono skutecznosc shotguna",1,2.f);
	}
	else if(letter==letters_i){
		letter=letters_j;
		current_char='j';
		current_charL='J';
		launchMissleTimeA=3.f;
		launchMissleTimeB=9.f;
		pushText(Vec2i(4,1),"Comicsy zwiekszyly szybkosc ladowania swoich pociskow (3-9 sekund)!",1,2.f);
		
		gunRandFactor=0.4f;
		pushText(Vec2i(4,2),"Zwiekszono skutecznosc shotguna",1,2.f);
	}
	else if(letter==letters_j){
		letter=letters_k;
		current_char='k';
		current_charL='K';
	}
	else if(letter==letters_k){
		letter=letters_l;
		current_char='l';
		current_charL='L';
		
		gunFireRate=0.5f;
		pushText(Vec2i(4,1),"Ulepszono fire rate broni (0.5s)",1,2.f);
	}
	else if(letter==letters_l){
		letter=letters_m;
		current_char='m';
		current_charL='M';
	}
	else if(letter==letters_m){
		letter=letters_n;
		current_char='n';
		current_charL='N';
		
		gunRandFactor=-0.2f;
		pushText(Vec2i(4,1),"Dostales najlepsze pociski!",1,2.f);
	}
	else if(letter==letters_n){
		letter=letters_o;
		current_char='o';
		current_charL='O';
	}
	else if(letter==letters_o){
		letter=letters_p;
		current_char='p';
		current_charL='P';
	}
	else if(letter==letters_p){
		letter=letters_r;
		current_char='r';
		current_charL='R';
	}
	else if(letter==letters_r){
		letter=letters_s;
		current_char='s';
		current_charL='S';
		
		gunFireRate=0.2f;
		pushText(Vec2i(4,1),"Ulepszono fire rate broni (0.2s)",1,2.f);
	}
	else if(letter==letters_s){
		letter=letters_t;
		current_char='t';
		current_charL='T';
		launchMissleTimeA=3.5f;
		launchMissleTimeB=8.f;
		pushText(Vec2i(4,1),"Comicsy zwiekszyly szybkosc ladowania swoich pociskow (3-8 sekund)!",1,2.f);
	}
	else if(letter==letters_t){
		letter=letters_plo;
		current_char='o';
		current_charL='O';
		
		gunPower=5.f;
		gunTime=0.5f;
		gunFireRate=0.1f;
		pushText(Vec2i(4,1),"Ulepszono fire rate broni (0.2s) i moc broni",1,2.f);
	}
	else if(letter==letters_plo){
		letter=letters_w;
		current_char='w';
		current_charL='W';
	}
	else if(letter==letters_w){
		letter=letters_y;
		current_char='y';
		current_charL='Y';
		launchMissleTimeA=1.f;
		launchMissleTimeB=6.f;
		pushText(Vec2i(4,1),"Comicsy zwiekszyly szybkosc ladowania swoich pociskow (1-6 sekund)!",1,2.f);
	}
	else if(letter==letters_y){
		letter=letters_z;
		current_char='z';
		current_charL='Z';
	}
	else if(letter==letters_z){
		pushText(Vec2i(10,10),"KONIEC - Zabiles wszystkie Comic Sansy!",1,5.f);
		pushText(Vec2i(10,11),"Dyplom za ukonczenie gry zostanie dostarczony poczta...",1,6.f);
		pushText(Vec2i(11,13),"game by .c41x",1,7.f);
		pushText(Vec2i(12,14),"http://jakubduracz.com",1,8.f);
		pushText(Vec2i(13,15),"jakubduracz@gmail.com",1,9.f);
		pushText(Vec2i(14,17),"game created using pointless engine (.less) ;)",1,10.f);
		pushFireworks(Vec2i(55,15),3,'a',true,true);
		pushFireworks(Vec2i(55,15),3,'a',true,true);
		pushFireworks(Vec2i(35,15),2,'a',true,true);
		pushFireworks(Vec2i(50,5),1,'a',true,true);
		pushFireworks(Vec2i(2,5),1,'a',true,true);
		pushFireworks(Vec2i(60,5),1,'a',true,true);
		pushFireworks(Vec2i(13,85),1,'a',true,true);
	}
	letter_x=rg.Int(-50,50);
	letter_y=rg.Int(-50,50);
	setLetterStartLife();
}
std::list<std::pair<Vec2i,float>> missles; //pociski wystrzelone przez przerazajace comic sansy
int hitsByMissles=0;
void pushMissle(){
	Vec2i pos(rg.Int(letter_x-16,letter_x+16),rg.Int(letter_y-16,letter_y+16));
	missles.push_back(std::pair<Vec2i,float>(pos,3.f));
}
void updateMissles(float dt){
	attron(COLOR_PAIR(10)| A_NORMAL);
	std::vector<std::list<std::pair<Vec2i,float>>::iterator> todel;
	for(std::list<std::pair<Vec2i,float>>::iterator it=missles.begin();it!=missles.end();++it){
		if(it->second<0.f){
			todel.push_back(it);
			if(cput(it->first.x,it->first.y))
				++hitsByMissles;
		}
		else{
			put(it->first.x,it->first.y,'A');
			if(it->second<1.f){
				put(it->first.x+1,it->first.y-1,current_charL);
				put(it->first.x+1,it->first.y+1,current_charL);
				put(it->first.x-1,it->first.y-1,current_charL);
				put(it->first.x-1,it->first.y+1,current_charL);
				
				put(it->first.x,it->first.y-2,current_charL);
				put(it->first.x+2,it->first.y,current_charL);
				put(it->first.x,it->first.y+2,current_charL);
				put(it->first.x-2,it->first.y,current_charL);
			}
			if(it->second<2.f){
				put(it->first.x+1,it->first.y,current_charL);
				put(it->first.x,it->first.y+1,current_charL);
				put(it->first.x,it->first.y-1,current_charL);
				put(it->first.x-1,it->first.y,current_charL);
			}
			it->second-=dt;
		}
	}
	for(int i=0;i<todel.size();++i)
		missles.erase(todel[i]);
}
void drawCockpit(){
	for(int i=1;i<W-1;++i){
		putnooff(0,i,'-');
		putnooff(H-1,i,'-');
	}
	for(int i=1;i<H-1;++i){
		putnooff(i,0,'|');
		putnooff(i,W-1,'|');
	}
	putnooff(0,0,'\\');
	putnooff(1,1,'\\');
	putnooff(2,2,'\\');
	
	putnooff(H-1,W-1,'\\');
	putnooff(H-2,W-2,'\\');
	putnooff(H-3,W-3,'\\');
	
	putnooff(0,W-1,'/');
	putnooff(1,W-2,'/');
	putnooff(2,W-3,'/');
	
	putnooff(H-1,0,'/');
	putnooff(H-2,1,'/');
	putnooff(H-3,2,'/');
	
	putnooff(H/2,W/2,'+');
	//putsnooff(11,10,DynamicString("X:")(W/2)(" Y:")(H/2));
	
	attron(COLOR_PAIR(9)| A_NORMAL);
	int progress=getLetterLife()*(H-2);
	if(progress<0)
		progress=0;
	else if(progress<4)
		progress=3;
	for(int i=2;i<progress;++i)
		putnooff(i,W-3,' ');
		
	attron(COLOR_PAIR(8)| A_NORMAL);
	progress=playerLife*(H-2);
	if(progress<0)
		progress=0;
	else if(progress<4)
		progress=3;
	for(int i=2;i<progress;++i)
		putnooff(i,2,' ');
		
	// pozycja litery
	attron(COLOR_PAIR(3)| A_NORMAL);
	Vec2f p(-ofx-letter_x+0,-ofy-letter_y+32);
	p.SetNormalize();
	p*=-5.f;
	putnooff(H/2+p.x,W/2+p.y,'.');
	
	if(godActive>0.f){
		attron(COLOR_PAIR(12)| A_NORMAL);
		Vec2f p(-ofx-gunOfDeathPos.x+12,-ofy-gunOfDeathPos.y+38); //??
		p.SetNormalize();
		p*=-5.f;
		putnooff(H/2+p.x,W/2+p.y,'.');
	}
}

int _tmain(int argc, _TCHAR* argv[])// SORT STL
{
    initscr();
    getmaxyx(stdscr,H,W);
    curs_set(0);
    noecho();
    start_color();
    init_pair(1, COLOR_WHITE, COLOR_BLACK);
    init_pair(2, COLOR_YELLOW+8, COLOR_BLACK);
    init_pair(3, COLOR_RED+8, COLOR_BLACK);
    init_pair(4, COLOR_WHITE+8, COLOR_BLACK);
    init_pair(5, COLOR_BLACK+8, COLOR_BLACK);
    
    init_pair(6, COLOR_WHITE, COLOR_BLACK);
    init_pair(7, COLOR_YELLOW+8, COLOR_BLACK);
    init_pair(8, COLOR_RED+8, COLOR_RED);
    init_pair(9, COLOR_BLUE+8, COLOR_BLUE);
    init_pair(10, COLOR_MAGENTA+8, COLOR_MAGENTA);
    init_pair(11, COLOR_GREEN+8, COLOR_GREEN);
    
    init_pair(12, COLOR_BLUE+8, COLOR_BLACK);
    
    printf("Studio 'Birds Can't Fly' przedstawia gre: ComicSansInvaders\n");
    system("pause");
/*
	mvaddch(10,10,'z');
	mvaddch(10,11,'Z');
	jLine(12,12,12,22);
	jLine(12,22,12,22);
	jLine(13,23,12,22);
	refresh();

	system("pause");*/

	nextLetter(true);
	pushExplosion(Vec2i(15,15),3,'z',true,true);
	pushExplosion(Vec2i(55,15),1,'a',false);
	//pushStars(Vec2i(W/2,H/2),'.',5);

	InputState is;
	is.Initialize(Vec2i(100,100),false,true);
	Timer t;
	
	// stars oddzielnie
	gfxStars *stars=new gfxStars(Vec2f(W/2,H/2),'.',60,5,40.f);
	stars->setTime(2.f*40.f);
	
	float xxx=0.f;
	float aa=0.f;
	float sc=0.1f;
	
	float timeMissle=5.f;
	float gunFireRateT=-1.f;
	
	float godIn=5.f;
	float turnOffgod=10.f;

    while(true)
    {
		transformL(0.f,1.f);
		aa+=0.01f;
		sc+=0.001f;
		if(sc>=1.f)
			sc=0.1f;
    
		//_sleep(50);
		const float mod=6.f;
		const float dt=t.GetElapsedSeconds();
		if(is.GetKey('W').Down){
			pushExplosion(Vec2i(15,15),3,'z');
		}
		if(is.GetKey('A').Down){
			//pushExplosion(Vec2i(50+16,-30+16),3,'A');
			//push
			//gunOfDeath=true;
		}
		if(is.GetKey(BTN_SPACE).Down){/*
			if(checkCollision(50+16,-30+16))
				pushExplosion(Vec2i(-30+16,50+16),1,'-');
			else pushExplosion(Vec2i(-30+16,50+16),1,'+');*/
			/*
			if(checkCollisionBlast(-ofx+H/2,-ofy+W/2)){
				pushExplosion(Vec2i(-ofy+W/2,-ofx+H/2),3,'a');
				transformL(0.f,1.f);
			}*/
			if(gunFireRateT<0.f){
				gunFireRateT=gunFireRate;
				pushPMissle(Vec2i(-ofx+H/2,-ofy+W/2));
			}
		}
		if(is.GetKey('S').Down){
			//if(xxx<=0.f){
			//	pushShake(2.f);
			//	xxx=2.f;
			//}
		}
		if(is.GetKey('Q').Down){
			break;
		}
		//if(is.GetKey('T').Down)
				//nextLetter();
			//pushText(Vec2i(10,1),"if(is.GetKey('T').Down)",1,5.f);
		if(xxx>0.f)
			xxx-=dt;
		
		Mouse m=is.GetMouseMovement();
		if(!lockSteering){
			const int maxoff=120;
			ofx-=m.Position.y/1;
			ofx=Clip(-maxoff,ofx,maxoff);
			ofy-=m.Position.x/1;
			ofy=Clip(-maxoff,ofy,maxoff);
		}
		
		if(gunFireRateT>=0.f)
			gunFireRateT-=dt;
		
		timeMissle-=dt;
		if(timeMissle<0.f){
			timeMissle=rg.Float(launchMissleTimeA,launchMissleTimeB);
			pushMissle();
		}
		if(hitsByMissles>0){
			playerLife-=0.05f;//*float(hitsByMissles);
			hitsByMissles=0;
			pushShake(2.f);
		}
    
        clear();
        attron(COLOR_PAIR(1)| A_NORMAL);
        clearTasks();
        tasks.sort(gfxTaskCmp());
        stars->update(dt);
		jDrawLetter();
        for(std::list<gfxTask*>::iterator it=tasks.begin();it!=tasks.end();++it)
			(*it)->update(dt/2.f);
			
		updateMissles(dt);
		updatePMissles(dt);
		
		godIn-=dt;
		if(godIn<0.f){
			gunOfDeathPos(rg.Int(-70,70),rg.Int(-70,70));
			godActive=10.f;
			godIn=rg.Float(gunOfDeathTime,gunOfDeathTime*2.f);
		}
		if(godActive>0.f){
			godActive-=dt;
			drawGunOfDeath();
			if(Abs(gunOfDeathPos.x+ofx-H/2)<4 && Abs(gunOfDeathPos.y+ofy-W/2)<4){
				gunOfDeath=true;
				godIn+=30.f;
				godActive=-1.f;
				pushText(Vec2i(4,1),"otrzymujesz GunOfComicDeath na 10 sekund!",1,2.f);
			}
		}
		if(gunOfDeath){
			pushText(Vec2i(4,2),DynamicString("GunOfComicDeath: ")(turnOffgod)("s"),1,1.f);
			turnOffgod-=dt;
			if(turnOffgod<0.f){
				gunOfDeath=false;
				turnOffgod=10.f;
			}
		}
		
		attron(COLOR_PAIR(1)| A_NORMAL);
		drawCockpit();
		//putsnooff(10,10,DynamicString("X:")(-ofy+W/2)(" Y:")(-ofx+H/2));
		//putsnooff(11,10,DynamicString("Life:")(getLetterLife()));
        refresh();
    }
    endwin();
    return 0;
}
